# Generated by xdrgen.py from ../../.xdr/ on Sat Sep 28 20:47:47 2019
KEY_TYPE_ED25519 = 0
KEY_TYPE_PRE_AUTH_TX = 1
KEY_TYPE_HASH_X = 2
CryptoKeyType = {0: "KEY_TYPE_ED25519", 1: "KEY_TYPE_PRE_AUTH_TX", 2: "KEY_TYPE_HASH_X"}
PUBLIC_KEY_TYPE_ED25519 = KEY_TYPE_ED25519
PublicKeyType = {KEY_TYPE_ED25519: "PUBLIC_KEY_TYPE_ED25519"}
SIGNER_KEY_TYPE_ED25519 = KEY_TYPE_ED25519
SIGNER_KEY_TYPE_PRE_AUTH_TX = KEY_TYPE_PRE_AUTH_TX
SIGNER_KEY_TYPE_HASH_X = KEY_TYPE_HASH_X
SignerKeyType = {
    KEY_TYPE_ED25519: "SIGNER_KEY_TYPE_ED25519",
    KEY_TYPE_PRE_AUTH_TX: "SIGNER_KEY_TYPE_PRE_AUTH_TX",
    KEY_TYPE_HASH_X: "SIGNER_KEY_TYPE_HASH_X",
}
SCP_ST_PREPARE = 0
SCP_ST_CONFIRM = 1
SCP_ST_EXTERNALIZE = 2
SCP_ST_NOMINATE = 3
SCPStatementType = {
    0: "SCP_ST_PREPARE",
    1: "SCP_ST_CONFIRM",
    2: "SCP_ST_EXTERNALIZE",
    3: "SCP_ST_NOMINATE",
}
STELLAR_VALUE_BASIC = 0
STELLAR_VALUE_SIGNED = 1
StellarValueType = {0: "STELLAR_VALUE_BASIC", 1: "STELLAR_VALUE_SIGNED"}
LEDGER_UPGRADE_VERSION = 1
LEDGER_UPGRADE_BASE_FEE = 2
LEDGER_UPGRADE_MAX_TX_SET_SIZE = 3
LEDGER_UPGRADE_BASE_RESERVE = 4
LedgerUpgradeType = {
    1: "LEDGER_UPGRADE_VERSION",
    2: "LEDGER_UPGRADE_BASE_FEE",
    3: "LEDGER_UPGRADE_MAX_TX_SET_SIZE",
    4: "LEDGER_UPGRADE_BASE_RESERVE",
}
METAENTRY = -1
LIVEENTRY = 0
DEADENTRY = 1
INITENTRY = 2
BucketEntryType = {-1: "METAENTRY", 0: "LIVEENTRY", 1: "DEADENTRY", 2: "INITENTRY"}
LEDGER_ENTRY_CREATED = 0
LEDGER_ENTRY_UPDATED = 1
LEDGER_ENTRY_REMOVED = 2
LEDGER_ENTRY_STATE = 3
LedgerEntryChangeType = {
    0: "LEDGER_ENTRY_CREATED",
    1: "LEDGER_ENTRY_UPDATED",
    2: "LEDGER_ENTRY_REMOVED",
    3: "LEDGER_ENTRY_STATE",
}
ASSET_TYPE_NATIVE = 0
ASSET_TYPE_CREDIT_ALPHANUM4 = 1
ASSET_TYPE_CREDIT_ALPHANUM12 = 2
AssetType = {
    0: "ASSET_TYPE_NATIVE",
    1: "ASSET_TYPE_CREDIT_ALPHANUM4",
    2: "ASSET_TYPE_CREDIT_ALPHANUM12",
}
THRESHOLD_MASTER_WEIGHT = 0
THRESHOLD_LOW = 1
THRESHOLD_MED = 2
THRESHOLD_HIGH = 3
ThresholdIndexes = {
    0: "THRESHOLD_MASTER_WEIGHT",
    1: "THRESHOLD_LOW",
    2: "THRESHOLD_MED",
    3: "THRESHOLD_HIGH",
}
ACCOUNT = 0
TRUSTLINE = 1
OFFER = 2
DATA = 3
LedgerEntryType = {0: "ACCOUNT", 1: "TRUSTLINE", 2: "OFFER", 3: "DATA"}
AUTH_REQUIRED_FLAG = 0x1
AUTH_REVOCABLE_FLAG = 0x2
AUTH_IMMUTABLE_FLAG = 0x4
AccountFlags = {
    0x1: "AUTH_REQUIRED_FLAG",
    0x2: "AUTH_REVOCABLE_FLAG",
    0x4: "AUTH_IMMUTABLE_FLAG",
}
MASK_ACCOUNT_FLAGS = 0x7
AUTHORIZED_FLAG = 1
TrustLineFlags = {1: "AUTHORIZED_FLAG"}
MASK_TRUSTLINE_FLAGS = 1
PASSIVE_FLAG = 1
OfferEntryFlags = {1: "PASSIVE_FLAG"}
MASK_OFFERENTRY_FLAGS = 1
ENVELOPE_TYPE_SCP = 1
ENVELOPE_TYPE_TX = 2
ENVELOPE_TYPE_AUTH = 3
ENVELOPE_TYPE_SCPVALUE = 4
EnvelopeType = {
    1: "ENVELOPE_TYPE_SCP",
    2: "ENVELOPE_TYPE_TX",
    3: "ENVELOPE_TYPE_AUTH",
    4: "ENVELOPE_TYPE_SCPVALUE",
}
CREATE_ACCOUNT = 0
PAYMENT = 1
PATH_PAYMENT_STRICT_RECEIVE = 2
MANAGE_SELL_OFFER = 3
CREATE_PASSIVE_SELL_OFFER = 4
SET_OPTIONS = 5
CHANGE_TRUST = 6
ALLOW_TRUST = 7
ACCOUNT_MERGE = 8
INFLATION = 9
MANAGE_DATA = 10
BUMP_SEQUENCE = 11
MANAGE_BUY_OFFER = 12
PATH_PAYMENT_STRICT_SEND = 13
OperationType = {
    0: "CREATE_ACCOUNT",
    1: "PAYMENT",
    2: "PATH_PAYMENT_STRICT_RECEIVE",
    3: "MANAGE_SELL_OFFER",
    4: "CREATE_PASSIVE_SELL_OFFER",
    5: "SET_OPTIONS",
    6: "CHANGE_TRUST",
    7: "ALLOW_TRUST",
    8: "ACCOUNT_MERGE",
    9: "INFLATION",
    10: "MANAGE_DATA",
    11: "BUMP_SEQUENCE",
    12: "MANAGE_BUY_OFFER",
    13: "PATH_PAYMENT_STRICT_SEND",
}
MEMO_NONE = 0
MEMO_TEXT = 1
MEMO_ID = 2
MEMO_HASH = 3
MEMO_RETURN = 4
MemoType = {
    0: "MEMO_NONE",
    1: "MEMO_TEXT",
    2: "MEMO_ID",
    3: "MEMO_HASH",
    4: "MEMO_RETURN",
}
MAX_OPS_PER_TX = 100
CREATE_ACCOUNT_SUCCESS = 0
CREATE_ACCOUNT_MALFORMED = -1
CREATE_ACCOUNT_UNDERFUNDED = -2
CREATE_ACCOUNT_LOW_RESERVE = -3
CREATE_ACCOUNT_ALREADY_EXIST = -4
CreateAccountResultCode = {
    0: "CREATE_ACCOUNT_SUCCESS",
    -1: "CREATE_ACCOUNT_MALFORMED",
    -2: "CREATE_ACCOUNT_UNDERFUNDED",
    -3: "CREATE_ACCOUNT_LOW_RESERVE",
    -4: "CREATE_ACCOUNT_ALREADY_EXIST",
}
PAYMENT_SUCCESS = 0
PAYMENT_MALFORMED = -1
PAYMENT_UNDERFUNDED = -2
PAYMENT_SRC_NO_TRUST = -3
PAYMENT_SRC_NOT_AUTHORIZED = -4
PAYMENT_NO_DESTINATION = -5
PAYMENT_NO_TRUST = -6
PAYMENT_NOT_AUTHORIZED = -7
PAYMENT_LINE_FULL = -8
PAYMENT_NO_ISSUER = -9
PaymentResultCode = {
    0: "PAYMENT_SUCCESS",
    -1: "PAYMENT_MALFORMED",
    -2: "PAYMENT_UNDERFUNDED",
    -3: "PAYMENT_SRC_NO_TRUST",
    -4: "PAYMENT_SRC_NOT_AUTHORIZED",
    -5: "PAYMENT_NO_DESTINATION",
    -6: "PAYMENT_NO_TRUST",
    -7: "PAYMENT_NOT_AUTHORIZED",
    -8: "PAYMENT_LINE_FULL",
    -9: "PAYMENT_NO_ISSUER",
}
PATH_PAYMENT_STRICT_RECEIVE_SUCCESS = 0
PATH_PAYMENT_STRICT_RECEIVE_MALFORMED = -1
PATH_PAYMENT_STRICT_RECEIVE_UNDERFUNDED = -2
PATH_PAYMENT_STRICT_RECEIVE_SRC_NO_TRUST = -3
PATH_PAYMENT_STRICT_RECEIVE_SRC_NOT_AUTHORIZED = -4
PATH_PAYMENT_STRICT_RECEIVE_NO_DESTINATION = -5
PATH_PAYMENT_STRICT_RECEIVE_NO_TRUST = -6
PATH_PAYMENT_STRICT_RECEIVE_NOT_AUTHORIZED = -7
PATH_PAYMENT_STRICT_RECEIVE_LINE_FULL = -8
PATH_PAYMENT_STRICT_RECEIVE_NO_ISSUER = -9
PATH_PAYMENT_STRICT_RECEIVE_TOO_FEW_OFFERS = -10
PATH_PAYMENT_STRICT_RECEIVE_OFFER_CROSS_SELF = -11
PATH_PAYMENT_STRICT_RECEIVE_OVER_SENDMAX = -12
PathPaymentStrictReceiveResultCode = {
    0: "PATH_PAYMENT_STRICT_RECEIVE_SUCCESS",
    -1: "PATH_PAYMENT_STRICT_RECEIVE_MALFORMED",
    -2: "PATH_PAYMENT_STRICT_RECEIVE_UNDERFUNDED",
    -3: "PATH_PAYMENT_STRICT_RECEIVE_SRC_NO_TRUST",
    -4: "PATH_PAYMENT_STRICT_RECEIVE_SRC_NOT_AUTHORIZED",
    -5: "PATH_PAYMENT_STRICT_RECEIVE_NO_DESTINATION",
    -6: "PATH_PAYMENT_STRICT_RECEIVE_NO_TRUST",
    -7: "PATH_PAYMENT_STRICT_RECEIVE_NOT_AUTHORIZED",
    -8: "PATH_PAYMENT_STRICT_RECEIVE_LINE_FULL",
    -9: "PATH_PAYMENT_STRICT_RECEIVE_NO_ISSUER",
    -10: "PATH_PAYMENT_STRICT_RECEIVE_TOO_FEW_OFFERS",
    -11: "PATH_PAYMENT_STRICT_RECEIVE_OFFER_CROSS_SELF",
    -12: "PATH_PAYMENT_STRICT_RECEIVE_OVER_SENDMAX",
}
PATH_PAYMENT_STRICT_SEND_SUCCESS = 0
PATH_PAYMENT_STRICT_SEND_MALFORMED = -1
PATH_PAYMENT_STRICT_SEND_UNDERFUNDED = -2
PATH_PAYMENT_STRICT_SEND_SRC_NO_TRUST = -3
PATH_PAYMENT_STRICT_SEND_SRC_NOT_AUTHORIZED = -4
PATH_PAYMENT_STRICT_SEND_NO_DESTINATION = -5
PATH_PAYMENT_STRICT_SEND_NO_TRUST = -6
PATH_PAYMENT_STRICT_SEND_NOT_AUTHORIZED = -7
PATH_PAYMENT_STRICT_SEND_LINE_FULL = -8
PATH_PAYMENT_STRICT_SEND_NO_ISSUER = -9
PATH_PAYMENT_STRICT_SEND_TOO_FEW_OFFERS = -10
PATH_PAYMENT_STRICT_SEND_OFFER_CROSS_SELF = -11
PATH_PAYMENT_STRICT_SEND_UNDER_DESTMIN = -12
PathPaymentStrictSendResultCode = {
    0: "PATH_PAYMENT_STRICT_SEND_SUCCESS",
    -1: "PATH_PAYMENT_STRICT_SEND_MALFORMED",
    -2: "PATH_PAYMENT_STRICT_SEND_UNDERFUNDED",
    -3: "PATH_PAYMENT_STRICT_SEND_SRC_NO_TRUST",
    -4: "PATH_PAYMENT_STRICT_SEND_SRC_NOT_AUTHORIZED",
    -5: "PATH_PAYMENT_STRICT_SEND_NO_DESTINATION",
    -6: "PATH_PAYMENT_STRICT_SEND_NO_TRUST",
    -7: "PATH_PAYMENT_STRICT_SEND_NOT_AUTHORIZED",
    -8: "PATH_PAYMENT_STRICT_SEND_LINE_FULL",
    -9: "PATH_PAYMENT_STRICT_SEND_NO_ISSUER",
    -10: "PATH_PAYMENT_STRICT_SEND_TOO_FEW_OFFERS",
    -11: "PATH_PAYMENT_STRICT_SEND_OFFER_CROSS_SELF",
    -12: "PATH_PAYMENT_STRICT_SEND_UNDER_DESTMIN",
}
MANAGE_SELL_OFFER_SUCCESS = 0
MANAGE_SELL_OFFER_MALFORMED = -1
MANAGE_SELL_OFFER_SELL_NO_TRUST = -2
MANAGE_SELL_OFFER_BUY_NO_TRUST = -3
MANAGE_SELL_OFFER_SELL_NOT_AUTHORIZED = -4
MANAGE_SELL_OFFER_BUY_NOT_AUTHORIZED = -5
MANAGE_SELL_OFFER_LINE_FULL = -6
MANAGE_SELL_OFFER_UNDERFUNDED = -7
MANAGE_SELL_OFFER_CROSS_SELF = -8
MANAGE_SELL_OFFER_SELL_NO_ISSUER = -9
MANAGE_SELL_OFFER_BUY_NO_ISSUER = -10
MANAGE_SELL_OFFER_NOT_FOUND = -11
MANAGE_SELL_OFFER_LOW_RESERVE = -12
ManageSellOfferResultCode = {
    0: "MANAGE_SELL_OFFER_SUCCESS",
    -1: "MANAGE_SELL_OFFER_MALFORMED",
    -2: "MANAGE_SELL_OFFER_SELL_NO_TRUST",
    -3: "MANAGE_SELL_OFFER_BUY_NO_TRUST",
    -4: "MANAGE_SELL_OFFER_SELL_NOT_AUTHORIZED",
    -5: "MANAGE_SELL_OFFER_BUY_NOT_AUTHORIZED",
    -6: "MANAGE_SELL_OFFER_LINE_FULL",
    -7: "MANAGE_SELL_OFFER_UNDERFUNDED",
    -8: "MANAGE_SELL_OFFER_CROSS_SELF",
    -9: "MANAGE_SELL_OFFER_SELL_NO_ISSUER",
    -10: "MANAGE_SELL_OFFER_BUY_NO_ISSUER",
    -11: "MANAGE_SELL_OFFER_NOT_FOUND",
    -12: "MANAGE_SELL_OFFER_LOW_RESERVE",
}
MANAGE_OFFER_CREATED = 0
MANAGE_OFFER_UPDATED = 1
MANAGE_OFFER_DELETED = 2
ManageOfferEffect = {
    0: "MANAGE_OFFER_CREATED",
    1: "MANAGE_OFFER_UPDATED",
    2: "MANAGE_OFFER_DELETED",
}
MANAGE_BUY_OFFER_SUCCESS = 0
MANAGE_BUY_OFFER_MALFORMED = -1
MANAGE_BUY_OFFER_SELL_NO_TRUST = -2
MANAGE_BUY_OFFER_BUY_NO_TRUST = -3
MANAGE_BUY_OFFER_SELL_NOT_AUTHORIZED = -4
MANAGE_BUY_OFFER_BUY_NOT_AUTHORIZED = -5
MANAGE_BUY_OFFER_LINE_FULL = -6
MANAGE_BUY_OFFER_UNDERFUNDED = -7
MANAGE_BUY_OFFER_CROSS_SELF = -8
MANAGE_BUY_OFFER_SELL_NO_ISSUER = -9
MANAGE_BUY_OFFER_BUY_NO_ISSUER = -10
MANAGE_BUY_OFFER_NOT_FOUND = -11
MANAGE_BUY_OFFER_LOW_RESERVE = -12
ManageBuyOfferResultCode = {
    0: "MANAGE_BUY_OFFER_SUCCESS",
    -1: "MANAGE_BUY_OFFER_MALFORMED",
    -2: "MANAGE_BUY_OFFER_SELL_NO_TRUST",
    -3: "MANAGE_BUY_OFFER_BUY_NO_TRUST",
    -4: "MANAGE_BUY_OFFER_SELL_NOT_AUTHORIZED",
    -5: "MANAGE_BUY_OFFER_BUY_NOT_AUTHORIZED",
    -6: "MANAGE_BUY_OFFER_LINE_FULL",
    -7: "MANAGE_BUY_OFFER_UNDERFUNDED",
    -8: "MANAGE_BUY_OFFER_CROSS_SELF",
    -9: "MANAGE_BUY_OFFER_SELL_NO_ISSUER",
    -10: "MANAGE_BUY_OFFER_BUY_NO_ISSUER",
    -11: "MANAGE_BUY_OFFER_NOT_FOUND",
    -12: "MANAGE_BUY_OFFER_LOW_RESERVE",
}
SET_OPTIONS_SUCCESS = 0
SET_OPTIONS_LOW_RESERVE = -1
SET_OPTIONS_TOO_MANY_SIGNERS = -2
SET_OPTIONS_BAD_FLAGS = -3
SET_OPTIONS_INVALID_INFLATION = -4
SET_OPTIONS_CANT_CHANGE = -5
SET_OPTIONS_UNKNOWN_FLAG = -6
SET_OPTIONS_THRESHOLD_OUT_OF_RANGE = -7
SET_OPTIONS_BAD_SIGNER = -8
SET_OPTIONS_INVALID_HOME_DOMAIN = -9
SetOptionsResultCode = {
    0: "SET_OPTIONS_SUCCESS",
    -1: "SET_OPTIONS_LOW_RESERVE",
    -2: "SET_OPTIONS_TOO_MANY_SIGNERS",
    -3: "SET_OPTIONS_BAD_FLAGS",
    -4: "SET_OPTIONS_INVALID_INFLATION",
    -5: "SET_OPTIONS_CANT_CHANGE",
    -6: "SET_OPTIONS_UNKNOWN_FLAG",
    -7: "SET_OPTIONS_THRESHOLD_OUT_OF_RANGE",
    -8: "SET_OPTIONS_BAD_SIGNER",
    -9: "SET_OPTIONS_INVALID_HOME_DOMAIN",
}
CHANGE_TRUST_SUCCESS = 0
CHANGE_TRUST_MALFORMED = -1
CHANGE_TRUST_NO_ISSUER = -2
CHANGE_TRUST_INVALID_LIMIT = -3
CHANGE_TRUST_LOW_RESERVE = -4
CHANGE_TRUST_SELF_NOT_ALLOWED = -5
ChangeTrustResultCode = {
    0: "CHANGE_TRUST_SUCCESS",
    -1: "CHANGE_TRUST_MALFORMED",
    -2: "CHANGE_TRUST_NO_ISSUER",
    -3: "CHANGE_TRUST_INVALID_LIMIT",
    -4: "CHANGE_TRUST_LOW_RESERVE",
    -5: "CHANGE_TRUST_SELF_NOT_ALLOWED",
}
ALLOW_TRUST_SUCCESS = 0
ALLOW_TRUST_MALFORMED = -1
ALLOW_TRUST_NO_TRUST_LINE = -2
ALLOW_TRUST_TRUST_NOT_REQUIRED = -3
ALLOW_TRUST_CANT_REVOKE = -4
ALLOW_TRUST_SELF_NOT_ALLOWED = -5
AllowTrustResultCode = {
    0: "ALLOW_TRUST_SUCCESS",
    -1: "ALLOW_TRUST_MALFORMED",
    -2: "ALLOW_TRUST_NO_TRUST_LINE",
    -3: "ALLOW_TRUST_TRUST_NOT_REQUIRED",
    -4: "ALLOW_TRUST_CANT_REVOKE",
    -5: "ALLOW_TRUST_SELF_NOT_ALLOWED",
}
ACCOUNT_MERGE_SUCCESS = 0
ACCOUNT_MERGE_MALFORMED = -1
ACCOUNT_MERGE_NO_ACCOUNT = -2
ACCOUNT_MERGE_IMMUTABLE_SET = -3
ACCOUNT_MERGE_HAS_SUB_ENTRIES = -4
ACCOUNT_MERGE_SEQNUM_TOO_FAR = -5
ACCOUNT_MERGE_DEST_FULL = -6
AccountMergeResultCode = {
    0: "ACCOUNT_MERGE_SUCCESS",
    -1: "ACCOUNT_MERGE_MALFORMED",
    -2: "ACCOUNT_MERGE_NO_ACCOUNT",
    -3: "ACCOUNT_MERGE_IMMUTABLE_SET",
    -4: "ACCOUNT_MERGE_HAS_SUB_ENTRIES",
    -5: "ACCOUNT_MERGE_SEQNUM_TOO_FAR",
    -6: "ACCOUNT_MERGE_DEST_FULL",
}
INFLATION_SUCCESS = 0
INFLATION_NOT_TIME = -1
InflationResultCode = {0: "INFLATION_SUCCESS", -1: "INFLATION_NOT_TIME"}
MANAGE_DATA_SUCCESS = 0
MANAGE_DATA_NOT_SUPPORTED_YET = -1
MANAGE_DATA_NAME_NOT_FOUND = -2
MANAGE_DATA_LOW_RESERVE = -3
MANAGE_DATA_INVALID_NAME = -4
ManageDataResultCode = {
    0: "MANAGE_DATA_SUCCESS",
    -1: "MANAGE_DATA_NOT_SUPPORTED_YET",
    -2: "MANAGE_DATA_NAME_NOT_FOUND",
    -3: "MANAGE_DATA_LOW_RESERVE",
    -4: "MANAGE_DATA_INVALID_NAME",
}
BUMP_SEQUENCE_SUCCESS = 0
BUMP_SEQUENCE_BAD_SEQ = -1
BumpSequenceResultCode = {0: "BUMP_SEQUENCE_SUCCESS", -1: "BUMP_SEQUENCE_BAD_SEQ"}
opINNER = 0
opBAD_AUTH = -1
opNO_ACCOUNT = -2
opNOT_SUPPORTED = -3
opTOO_MANY_SUBENTRIES = -4
opEXCEEDED_WORK_LIMIT = -5
OperationResultCode = {
    0: "opINNER",
    -1: "opBAD_AUTH",
    -2: "opNO_ACCOUNT",
    -3: "opNOT_SUPPORTED",
    -4: "opTOO_MANY_SUBENTRIES",
    -5: "opEXCEEDED_WORK_LIMIT",
}
txSUCCESS = 0
txFAILED = -1
txTOO_EARLY = -2
txTOO_LATE = -3
txMISSING_OPERATION = -4
txBAD_SEQ = -5
txBAD_AUTH = -6
txINSUFFICIENT_BALANCE = -7
txNO_ACCOUNT = -8
txINSUFFICIENT_FEE = -9
txBAD_AUTH_EXTRA = -10
txINTERNAL_ERROR = -11
TransactionResultCode = {
    0: "txSUCCESS",
    -1: "txFAILED",
    -2: "txTOO_EARLY",
    -3: "txTOO_LATE",
    -4: "txMISSING_OPERATION",
    -5: "txBAD_SEQ",
    -6: "txBAD_AUTH",
    -7: "txINSUFFICIENT_BALANCE",
    -8: "txNO_ACCOUNT",
    -9: "txINSUFFICIENT_FEE",
    -10: "txBAD_AUTH_EXTRA",
    -11: "txINTERNAL_ERROR",
}
ERR_MISC = 0
ERR_DATA = 1
ERR_CONF = 2
ERR_AUTH = 3
ERR_LOAD = 4
ErrorCode = {0: "ERR_MISC", 1: "ERR_DATA", 2: "ERR_CONF", 3: "ERR_AUTH", 4: "ERR_LOAD"}
IPv4 = 0
IPv6 = 1
IPAddrType = {0: "IPv4", 1: "IPv6"}
ERROR_MSG = 0
AUTH = 2
DONT_HAVE = 3
GET_PEERS = 4
PEERS = 5
GET_TX_SET = 6
TX_SET = 7
TRANSACTION = 8
GET_SCP_QUORUMSET = 9
SCP_QUORUMSET = 10
SCP_MESSAGE = 11
GET_SCP_STATE = 12
HELLO = 13
MessageType = {
    0: "ERROR_MSG",
    2: "AUTH",
    3: "DONT_HAVE",
    4: "GET_PEERS",
    5: "PEERS",
    6: "GET_TX_SET",
    7: "TX_SET",
    8: "TRANSACTION",
    9: "GET_SCP_QUORUMSET",
    10: "SCP_QUORUMSET",
    11: "SCP_MESSAGE",
    12: "GET_SCP_STATE",
    13: "HELLO",
}



from typing import Union, Coroutine, Any, Dict
from urllib.parse import urljoin

from .account import Account
from .asset import Asset
from .call_builder.accounts_call_builder import AccountsCallBuilder
from .call_builder.assets_call_builder import AssetsCallBuilder
from .call_builder.effects_call_builder import EffectsCallBuilder
from .call_builder.fee_stats_call_builder import FeeStatsCallBuilder
from .call_builder.ledgers_call_builder import LedgersCallBuilder
from .call_builder.offers_call_builder import OffersCallBuilder
from .call_builder.operations_call_builder import OperationsCallBuilder
from .call_builder.orderbook_call_builder import OrderbookCallBuilder
from .call_builder.paths_call_builder import PathsCallBuilder
from .call_builder.payments_call_builder import PaymentsCallBuilder
from .call_builder.root_call_builder import RootCallBuilder
from .call_builder.trades_aggregation_call_builder import TradeAggregationsCallBuilder
from .call_builder.trades_call_builder import TradesCallBuilder
from .call_builder.transactions_call_builder import TransactionsCallBuilder
from .client.base_async_client import BaseAsyncClient
from .client.base_sync_client import BaseSyncClient
from .client.requests_client import RequestsClient
from .exceptions import ValueError, raise_request_exception
from .transaction_envelope import TransactionEnvelope

__all__ = ["Server"]


class Server:
    """Server handles the network connection to a `Horizon <https://www.stellar.org/developers/horizon/reference/>`_
    instance and exposes an interface for requests to that instance.

    Here we need to talk about the **client** parameter, if you do not specify the client, we will use
    the :class:`stellar_sdk.client.requests_client.RequestsClient` instance by default, it is a synchronous HTTPClient,
    you can also specify an asynchronous HTTP Client,
    for example: :class:`stellar_sdk.client.aiohttp_client.AiohttpClient`. If you use a synchronous client,
    then all requests are synchronous. If you use an asynchronous client,
    then all requests are asynchronous. The choice is in your hands.

    :param horizon_url: Horizon Server URL (ex. `https://horizon-testnet.stellar.org`)
    :param client: Http Client used to send the request
    :raises:
        :exc:`ValueError: <stellar_sdk.exceptions.ValueError:>`
    """

    def __init__(
        self,
        horizon_url: str = "https://horizon-testnet.stellar.org/",
        client: Union[BaseAsyncClient, BaseSyncClient] = None,
    ) -> None:
        self.horizon_url: str = horizon_url

        if not client:
            client = RequestsClient()
        self._client: Union[BaseAsyncClient, BaseSyncClient] = client

        if isinstance(self._client, BaseAsyncClient):
            self.__async: bool = True
        elif isinstance(self._client, BaseSyncClient):
            self.__async = False
        else:
            raise ValueError(
                "This `client` class should be an instance "
                "of `stellar_sdk.client.base_async_client.BaseAsyncClient` "
                "or `stellar_sdk.client.base_sync_client.BaseSyncClient`."
            )

    def submit_transaction(
        self, transaction_envelope: Union[TransactionEnvelope, str]
    ) -> Union[Dict[str, Any], Coroutine[Any, Any, Dict[str, Any]]]:
        """Submits a transaction to the network.

        :param transaction_envelope: :class:`stellar_sdk.transaction_envelope.TransactionEnvelope` object
            or base64 encoded xdr
        :return: the response from horizon
        """
        xdr = transaction_envelope
        if isinstance(transaction_envelope, TransactionEnvelope):
            xdr = transaction_envelope.to_xdr()

        data = {"tx": xdr}
        url = urljoin(self.horizon_url, "/transactions")
        if self.__async:
            return self.__submit_transaction_async(url, data)
        return self.__submit_transaction_sync(url, data)

    def __submit_transaction_sync(
        self, url: str, data: Dict[str, str]
    ) -> Dict[str, Any]:
        resp = self._client.post(url=url, data=data)
        raise_request_exception(resp)
        return resp.json()

    async def __submit_transaction_async(
        self, url: str, data: Dict[str, str]
    ) -> Dict[str, Any]:
        resp = await self._client.post(url=url, data=data)
        raise_request_exception(resp)
        return resp.json()

    def root(self) -> RootCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.RootCallBuilder` object configured
            by a current Horizon server configuration.
        """
        return RootCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def accounts(self) -> AccountsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.AccountsCallBuilder` object configured
            by a current Horizon server configuration.
        """
        return AccountsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def assets(self) -> AssetsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.AssetsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return AssetsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def effects(self) -> EffectsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.EffectsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return EffectsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def fee_stats(self) -> FeeStatsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.FeeStatsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return FeeStatsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def ledgers(self) -> LedgersCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.LedgersCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return LedgersCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def offers(self, account_id: str) -> OffersCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.OffersCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return OffersCallBuilder(
            horizon_url=self.horizon_url, client=self._client, account_id=account_id
        )

    def operations(self) -> OperationsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.OperationsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return OperationsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def orderbook(self, selling: Asset, buying: Asset) -> OrderbookCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.OrderbookCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return OrderbookCallBuilder(
            horizon_url=self.horizon_url,
            client=self._client,
            buying=buying,
            selling=selling,
        )

    def paths(
        self,
        source_account: str,
        destination_account: str,
        destination_asset: Asset,
        destination_amount: str,
    ) -> PathsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.PathsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return PathsCallBuilder(
            horizon_url=self.horizon_url,
            client=self._client,
            source_account=source_account,
            destination_account=destination_account,
            destination_asset=destination_asset,
            destination_amount=destination_amount,
        )

    def payments(self) -> PaymentsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.PaymentsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return PaymentsCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def trade_aggregations(
        self,
        base: Asset,
        counter: Asset,
        resolution: int,
        start_time: int = None,
        end_time: int = None,
        offset: int = None,
    ) -> TradeAggregationsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.TradeAggregationsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return TradeAggregationsCallBuilder(
            horizon_url=self.horizon_url,
            client=self._client,
            base=base,
            counter=counter,
            start_time=start_time,
            end_time=end_time,
            resolution=resolution,
            offset=offset,
        )

    def trades(self) -> TradesCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.TradesCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return TradesCallBuilder(horizon_url=self.horizon_url, client=self._client)

    def transactions(self) -> TransactionsCallBuilder:
        """
        :return: New :class:`stellar_sdk.call_builder.TransactionsCallBuilder` object configured by
            a current Horizon server configuration.
        """
        return TransactionsCallBuilder(
            horizon_url=self.horizon_url, client=self._client
        )

    def load_account(
        self, account_id: str
    ) -> Union[Account, Coroutine[Any, Any, Account]]:
        """Fetches an account's most current state in the ledger and then creates
        and returns an :class:`stellar_sdk.account.Account` object.

        :param account_id: The account to load.
        :return: an :class:`stellar_sdk.account.Account` object.
        :raises:
            :exc:`ConnectionError <stellar_sdk.exceptions.ConnectionError>`
            :exc:`NotFoundError <stellar_sdk.exceptions.NotFoundError>`
            :exc:`BadRequestError <stellar_sdk.exceptions.BadRequestError>`
            :exc:`BadResponseError <stellar_sdk.exceptions.BadResponseError>`
            :exc:`UnknownRequestError <stellar_sdk.exceptions.UnknownRequestError>`
        """
        if self.__async:
            return self.__load_account_async(account_id)
        return self.__load_account_sync(account_id)

    async def __load_account_async(self, account_id: str) -> Account:
        resp = await self.accounts().account_id(account_id=account_id).call()
        sequence = int(resp["sequence"])
        return Account(account_id=account_id, sequence=sequence)

    def __load_account_sync(self, account_id: str) -> Account:
        resp = self.accounts().account_id(account_id=account_id).call()
        sequence = int(resp["sequence"])
        return Account(account_id=account_id, sequence=sequence)

    def fetch_base_fee(self) -> Union[int, Coroutine[Any, Any, int]]:
        """Fetch the base fee. Since this hits the server, if the server call fails,
        you might get an error. You should be prepared to use a default value if that happens.

        :return: the base fee
        :raises:
            :exc:`ConnectionError <stellar_sdk.exceptions.ConnectionError>`
            :exc:`NotFoundError <stellar_sdk.exceptions.NotFoundError>`
            :exc:`BadRequestError <stellar_sdk.exceptions.BadRequestError>`
            :exc:`BadResponseError <stellar_sdk.exceptions.BadResponseError>`
            :exc:`UnknownRequestError <stellar_sdk.exceptions.UnknownRequestError>`
        """
        if self.__async:
            return self.__fetch_base_fee_async()
        return self.__fetch_base_fee_sync()

    def __fetch_base_fee_sync(self) -> int:
        latest_ledger = self.ledgers().order(desc=True).limit(1).call()
        base_fee = self.__handle_base_fee(latest_ledger)
        return base_fee

    async def __fetch_base_fee_async(self) -> int:
        latest_ledger = await self.ledgers().order(desc=True).limit(1).call()
        base_fee = self.__handle_base_fee(latest_ledger)
        return base_fee

    def __handle_base_fee(self, latest_ledger: dict) -> int:
        base_fee = 100
        if (
            latest_ledger["_embedded"]
            and latest_ledger["_embedded"]["records"]
            and latest_ledger["_embedded"]["records"][0]
        ):
            base_fee = int(
                latest_ledger["_embedded"]["records"][0]["base_fee_in_stroops"]
            )
        return base_fee

    def close(self) -> None:
        """Close underlying connector.

        Release all acquired resources.
        """
        if self.__async:
            return self.__close_async()
        else:
            return self.__close_sync()

    async def __close_async(self) -> None:
        await self._client.close()

    def __close_sync(self) -> None:
        self._client.close()

    async def __aenter__(self) -> "Server":
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.close()

    def __enter__(self) -> "Server":
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()



import abc
from typing import Union

from .exceptions import MemoInvalidException
from .xdr import Xdr

__all__ = ["Memo", "NoneMemo", "TextMemo", "IdMemo", "HashMemo", "ReturnHashMemo"]


class Memo(object, metaclass=abc.ABCMeta):
    """The :class:`Memo` object, which represents the base class for memos for
    use with Stellar transactions.

    The memo for a transaction contains optional extra information about the
    transaction taking place. It is the responsibility of the client to
    interpret this value.

    See the following implementations that serve a more practical use with the
    library:

    * :class:`NoneMemo` - No memo.
    * :class:`TextMemo` - A string encoded using either ASCII or UTF-8, up to
        28-bytes long.
    * :class:`IdMemo` - A 64 bit unsigned integer.
    * :class:`HashMemo` - A 32 byte hash.
    * :class:`RetHashMemo` -  A 32 byte hash intended to be interpreted as the
        hash of the transaction the sender is refunding.

    See `Stellar's documentation on Transactions
    <https://www.stellar.org/developers/guides/concepts/transactions.html#memo>`__
    for more information on how memos are used within transactions, as well as
    information on the available types of memos.

    """

    @abc.abstractmethod
    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`Memo`."""

    @staticmethod
    def from_xdr_object(xdr_obj: Xdr.types.Memo) -> "Memo":
        """Returns an Memo object from XDR memo object."""

        xdr_types = {
            Xdr.const.MEMO_TEXT: TextMemo,
            Xdr.const.MEMO_ID: IdMemo,
            Xdr.const.MEMO_HASH: HashMemo,
            Xdr.const.MEMO_RETURN: ReturnHashMemo,
            Xdr.const.MEMO_NONE: NoneMemo,
        }

        # TODO: Maybe we should raise Key Error here
        memo_cls = xdr_types.get(xdr_obj.type, NoneMemo)
        return memo_cls.from_xdr_object(xdr_obj)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.to_xdr_object().to_xdr() == other.to_xdr_object().to_xdr()


class NoneMemo(Memo):
    """The :class:`NoneMemo`, which represents no memo for a transaction."""

    @classmethod
    def from_xdr_object(cls, xdr_obj: Xdr.types.Memo) -> "NoneMemo":
        """Returns an :class:`NoneMemo` object from XDR memo object."""

        return cls()

    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`NoneMemo`."""
        return Xdr.types.Memo(type=Xdr.const.MEMO_NONE)

    def __str__(self):
        return "<NoneMemo>"


class TextMemo(Memo):
    """The :class:`TextMemo`, which represents MEMO_TEXT in a transaction.

    :param text: A string encoded using either ASCII or UTF-8, up to
        28-bytes long.
    :type text: str, bytes
    :raises:
        :exc:`MemoInvalidException: <stellar_sdk.exceptions.MemoInvalidException:>`

    """

    def __init__(self, text: Union[str, bytes]) -> None:
        if not isinstance(text, (str, bytes)):
            raise MemoInvalidException(
                "TextMemo expects string or bytes type got a {}".format(type(text))
            )

        self.memo_text = text
        if not isinstance(text, bytes):
            self.memo_text = bytearray(text, encoding="utf-8")

        length = len(self.memo_text)
        if length > 28:
            raise MemoInvalidException(
                "Text should be <= 28 bytes (ascii encoded), got {:d} bytes.".format(
                    length
                )
            )

    @classmethod
    def from_xdr_object(cls, xdr_obj: Xdr.types.Memo) -> "TextMemo":
        """Returns an :class:`TextMemo` object from XDR memo object."""

        return cls(bytes(xdr_obj.switch))

    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`TextMemo`."""
        return Xdr.types.Memo(type=Xdr.const.MEMO_TEXT, text=self.memo_text)

    def __str__(self):
        return "<TextMemo [memo={memo}]>".format(memo=self.memo_text)


class IdMemo(Memo):
    """The :class:`IdMemo` which represents MEMO_ID in a transaction.

    :param int memo_id: A 64 bit unsigned integer.
    :raises:
        :exc:`MemoInvalidException: <stellar_sdk.exceptions.MemoInvalidException:>`

    """

    def __init__(self, memo_id: int) -> None:
        if memo_id < 0 or memo_id > 2 ** 64 - 1:
            raise MemoInvalidException(
                "IdMemo is an unsigned 64-bit integer and the max valid value is 18446744073709551615."
            )
        self.memo_id = memo_id

    @classmethod
    def from_xdr_object(cls, xdr_obj: Xdr.types.Memo) -> "IdMemo":
        """Returns an :class:`IdMemo` object from XDR memo object."""

        return cls(xdr_obj.switch)

    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`IdMemo`."""
        return Xdr.types.Memo(type=Xdr.const.MEMO_ID, id=self.memo_id)

    def __str__(self):
        return "<IdMemo [memo={memo}]>".format(memo=self.memo_id)


class HashMemo(Memo):
    """The :class:`HashMemo` which represents MEMO_HASH in a transaction.

    :param memo_hash: A 32 byte hash.
    :raises:
        :exc:`MemoInvalidException: <stellar_sdk.exceptions.MemoInvalidException:>`
    """

    def __init__(self, memo_hash: bytes) -> None:
        length = len(memo_hash)
        if length > 32:
            raise MemoInvalidException(
                "HashMemo can contain 32 bytes at max, got {:d} bytes".format(length)
            )

        self.memo_hash = memo_hash

    @classmethod
    def from_xdr_object(cls, xdr_obj: Xdr.types.Memo) -> "HashMemo":
        """Returns an :class:`HashMemo` object from XDR memo object."""

        return cls(xdr_obj.switch)

    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`HashMemo`."""
        return Xdr.types.Memo(type=Xdr.const.MEMO_HASH, hash=self.memo_hash)

    def __str__(self):
        return "<HashMemo [memo={memo}]>".format(memo=self.memo_hash)


class ReturnHashMemo(Memo):
    """The :class:`ReturnHashMemo` which represents MEMO_RETURN in a transaction.

    MEMO_RETURN is typically used with refunds/returns over the network - it is
    a 32 byte hash intended to be interpreted as the hash of the transaction
    the sender is refunding.

    :param memo_return: A 32 byte hash intended to be interpreted as the
        hash of the transaction the sender is refunding.
    :raises:
        :exc:`MemoInvalidException: <stellar_sdk.exceptions.MemoInvalidException:>`
    """

    def __init__(self, memo_return: bytes) -> None:
        length = len(memo_return)
        if length > 32:
            raise MemoInvalidException(
                "ReturnHashMemo can contain 32 bytes at max, got {:d} bytes".format(
                    length
                )
            )

        self.memo_return = memo_return

    @classmethod
    def from_xdr_object(cls, xdr_obj: Xdr.types.Memo) -> "ReturnHashMemo":
        """Returns an :class:`ReturnHashMemo` object from XDR memo object."""
        return cls(xdr_obj.switch)

    def to_xdr_object(self) -> Xdr.types.Memo:
        """Creates an XDR Memo object that represents this :class:`ReturnHashMemo`."""
        return Xdr.types.Memo(type=Xdr.const.MEMO_RETURN, retHash=self.memo_return)

    def __str__(self):
        return "<ReturnHashMemo [memo={memo}]>".format(memo=self.memo_return)



from typing import List, Union

from .exceptions import ValueError
from .keypair import Keypair
from .memo import NoneMemo, Memo
from .operation.operation import Operation
from .xdr import Xdr
from .strkey import StrKey
from .time_bounds import TimeBounds
from .utils import pack_xdr_array, unpack_xdr_array

__all__ = ["Transaction"]


class Transaction:
    """The :class:`Transaction` object, which represents a transaction
    on Stellar's network.

    A transaction contains a list of operations, which are all executed
    in order as one ACID transaction, along with an
    associated source account, fee, account sequence number, list of
    signatures, both an optional memo and an optional TimeBounds. Typically a
    :class:`Transaction` is placed in a :class:`TransactionEnvelope
    <stellar_sdk.transaction_envelope.TransactionEnvelope>` which is
    then signed before being sent over the network.

    For more information on Transactions in Stellar, see `Stellar's guide
    on transactions`_.

    .. _Stellar's guide on transactions:
        https://www.stellar.org/developers/guides/concepts/transactions.html

    :param source: the source account for the transaction.
    :param sequence: The sequence number for the transaction.
    :param fee: The fee amount for the transaction, which should equal
          FEE (currently 100 stroops) multiplied by the number of
          operations in the transaction. See `Stellar's latest documentation
          on fees
          <https://www.stellar.org/developers/guides/concepts/fees.html#transaction-fee>`_
          for more information.
    :param operations: A list of operations objects (typically its
          subclasses as defined in :mod:`stellar_sdk.operation.Operation`.
    :param time_bounds: The timebounds for the validity of this transaction.
    :param memo: The memo being sent with the transaction, being
          represented as one of the subclasses of the
          :class:`Memo <stellar_sdk.memo.Memo>` object.
    """

    def __init__(
        self,
        source: Keypair,
        sequence: int,
        fee: int,
        operations: List[Operation],
        memo: Union[Memo] = None,
        time_bounds: TimeBounds = None,
    ) -> None:

        # if not operations:
        #     raise ValueError("At least one operation required.")

        if not memo:
            memo = NoneMemo()

        self.source = source
        self.sequence = sequence
        self.operations = operations
        self.memo = memo
        self.fee = fee
        self.time_bounds = time_bounds

    def to_xdr_object(self) -> Xdr.types.Transaction:
        """Get an XDR object representation of this :class:`Transaction`.

        :return: XDR Transaction object
        """
        source_account = self.source.xdr_account_id()
        memo = self.memo.to_xdr_object()
        operations = [operation.to_xdr_object() for operation in self.operations]
        time_bounds: List[TimeBounds] = []
        if self.time_bounds:
            time_bounds = pack_xdr_array(self.time_bounds.to_xdr_object())
        ext = Xdr.nullclass()
        ext.v = 0
        return Xdr.types.Transaction(
            source_account, self.fee, self.sequence, time_bounds, memo, operations, ext
        )

    @classmethod
    def from_xdr_object(cls, tx_xdr_object) -> "Transaction":
        """Create a new :class:`Transaction` from an XDR object.

        :param tx_xdr_object: The XDR object that represents a transaction.

        :return: A new :class:`Transaction` object from the given XDR Transaction object.
        """
        source = Keypair.from_public_key(
            StrKey.encode_ed25519_public_key(tx_xdr_object.sourceAccount.ed25519)
        )
        sequence = tx_xdr_object.seqNum
        fee = tx_xdr_object.fee
        time_bounds_in_xdr = tx_xdr_object.timeBounds
        time_bounds = None
        if time_bounds_in_xdr:
            time_bounds = TimeBounds.from_xdr_object(
                unpack_xdr_array(time_bounds_in_xdr)
            )

        memo = Memo.from_xdr_object(tx_xdr_object.memo)
        operations = list(map(Operation.from_xdr_object, tx_xdr_object.operations))
        return cls(
            source=source,
            sequence=sequence,
            time_bounds=time_bounds,
            memo=memo,
            fee=fee,
            operations=operations,
        )



from .strkey import StrKey

__all__ = ["Account"]


class Account:
    """The :class:`Account` object, which represents represents a single
    account in Stellar network and its sequence number.

    Account tracks the sequence number as it is used
    by :class:`TransactionBuilder <stellar_sdk.transaction_builder.TransactionBuilder>`

    See `Accounts`_ For more information about the formats used for asset codes and how issuers
    work on Stellar's network,

    :param account_id: Account ID of the
        account (ex. `GB3KJPLFUYN5VL6R3GU3EGCGVCKFDSD7BEDX42HWG5BWFKB3KQGJJRMA`)
    :param sequence: sequence current sequence number of the account
    :raises:
        :exc:`Ed25519PublicKeyInvalidError: <stellar_sdk.exceptions.Ed25519PublicKeyInvalidError:>`

    .. _Accounts:
        https://stellar.org/developers/learn/concepts/accounts.html
    """

    def __init__(self, account_id: str, sequence: int) -> None:
        StrKey.decode_ed25519_public_key(account_id)

        self.account_id = account_id
        self.sequence = sequence

    def increment_sequence_number(self) -> None:
        """
        Increments sequence number in this object by one.
        """
        self.sequence += 1

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.account_id == other.account_id and self.sequence == other.sequence

    def __str__(self):
        return "<Account [account_id={account_id}, sequence={sequence}]>".format(
            account_id=self.account_id, sequence=self.sequence
        )
